<?php

namespace Drupal\xnttjson\Plugin\ExternalEntities\StorageClient;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\external_entities\ResponseDecoder\ResponseDecoderFactoryInterface;
use Drupal\xnttfiles\Plugin\ExternalEntities\StorageClient\FileClientBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * External entities storage client for JSON files.
 *
 * @ExternalEntityStorageClient(
 *   id = "xnttjson",
 *   label = @Translation("JSON Files"),
 *   description = @Translation("Retrieves and stores records in JSON files.")
 * )
 */
class JsonFiles extends FileClientBase {

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    TranslationInterface $string_translation,
    ResponseDecoderFactoryInterface $response_decoder_factory,
    ConfigFactory $config_factory,
    MessengerInterface $messenger,
    LoggerChannelInterface $logger,
    CacheBackendInterface $cache
  ) {
    parent::__construct(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $string_translation,
      $response_decoder_factory,
      $config_factory,
      $messenger,
      $logger,
      $cache
    );
    // Defaults.
    $this->fileType = 'JSON';
    $this->fileTypePlural = 'JSON';
    $this->fileTypeCap = 'JSON';
    $this->fileTypeCapPlural = 'JSON';
    $this->fileExtension = '.json';
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('string_translation'),
      $container->get('external_entities.response_decoder_factory'),
      $container->get('config.factory'),
      $container->get('messenger'),
      $container->get('logger.channel.xnttjson'),
      $container->get('cache.default')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function parseFile(string $file_path) :array {
    $data = [];
    $idf = $this->getIdField();
    $json_raw = file_get_contents($file_path);
    if (!empty($json_raw)) {
      $json = json_decode($json_raw, true);
      if (NULL == $json) {
        $this->logger->error(
          'Failed to parse JSON file "'
          . $file_path
          . '": '
          . json_last_error_msg()
        );
      }
      else {
        if ($this->configuration['multi_records']) {
          $columns = [];
          // Multiple records, we assume we have an array.
          if (is_array($json)) {
            // Process each entity.
            foreach ($json as $entity) {
              if (!empty($entity[$idf])) {
                $data[$entity[$idf]] = $entity;
              }
              $columns += array_flip(array_keys($entity));
            }
          }
          $data[''] = array_keys($columns);
        }
        else {
          // Single record. We assume we have one entity.
          if (is_array($json) && (!empty($json[$idf]))) {
            $data[$json[$idf]] = $json;
            $data[''] = array_keys($json);
          }
        }
      }
    }

    return $data;
  }

  /**
   * {@inheritdoc}
   */
  protected function generateRawData(array $entities_data) :string {
    // Don't save columns.
    $columns = array_flip($entities_data['']);
    unset($entities_data['']);
    if ($this->configuration['multi_records']) {
      // Multiple values.
      $json_entities = [];
      foreach ($entities_data as $id => $entity) {
        $json_entities[] = array_intersect_key($entity, $columns);
      }
      $json_raw = json_encode($json_entities);
    }
    else {
      // Single value.
      $entity = array_shift($entities_data);
      $json_entity = array_intersect_key($entity, $columns);
      $json_raw = json_encode($json_entity);
    }
    // Write records.
    return $json_raw;
  }

}
